#include <iostream>

// LibXML2 Includes
extern "C" {
#include <libxml/parser.h>
#include <libxml/tree.h>
}

using namespace std;

void outputElementNames(xmlNodePtr node, int tabLevel){
    // Recover element name
    const xmlChar *elementName = node->name;
    
    // Output indentation
    for(int i = 0; i < tabLevel; i++) { cout << "  "; }

    // Output element name
    cout << "Found element: " << (const char*) elementName << endl;

    // Recursively call outputElementNames() for child elements
    for(xmlNodePtr child = node->children;
        child;
        child = child->next){
        outputElementNames(child, tabLevel + 1);
    }
}

int main(int argc, char* argv[]){

    // Runtime check for correct LibXML2 version
    LIBXML_TEST_VERSION

    if(argc != 2){
        cerr << "Usage: example <FILEPATH>" << endl;
        return 1;
    }

    // Parse input file
    xmlDocPtr domRoot = xmlReadFile(argv[1], NULL, 0);

    // Check if parsing was successful
    if(not domRoot){
        cerr << "FAILED: Bad input file" << endl;
        return 1;
    }

    // Recover root node
    xmlNodePtr rootNode = xmlDocGetRootElement(domRoot);

    // Recursively output element names
    outputElementNames(rootNode, 0);

    cerr << "SUCCESS" << endl;

    // Free document
    xmlFreeDoc(domRoot);

    return 0;
}
